const express = require('express');
const helmet = require('helmet');
const xss = require('xss-clean');
require('express-async-errors');
const mongoSanitize = require('express-mongo-sanitize');
const compression = require('compression');
const cors = require('cors');
const mongoose = require('mongoose');
const config = require('./config/config');
const logger = require('./config/logger');
const loggingMiddleware = require('./middlewares/logging');
const errorHandler = require('./middlewares/errorHandler');
const emailService = require('./services/emailService');
const passport = require('./config/passport');
const { authLimiter } = require('./middlewares/rateLimit');
const blogsroute = require('./routes/blogsroute');
const authroute = require('./routes/authroute');
const app = express();

app.use(express.static('public'));
app.use(loggingMiddleware);
app.use(helmet());
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(xss());
app.use(mongoSanitize());
app.use(compression());
app.use(cors());
app.options('*', cors());
app.use(passport.initialize());


if (config.env === 'production') {
  app.use('/api', authLimiter);
}


app.use('/blogs', blogsroute);
app.use('/api', authroute);
app.use(errorHandler);


const startServer = async () => {
  try {
 
    await mongoose.connect(config.mongoose.url, config.mongoose.options);
    logger.info('Connected to MongoDB');

   
    await emailService.initializeEmailService();
    logger.info('Email service initialized');


    app.listen(config.port, () => {
      logger.info(`Server is running on port ${config.port}`);
    });
  } catch (error) {
    logger.error('Error during startup:', error);
    process.exit(1);
  }
};

startServer();


process.on('uncaughtException', (error) => {
  logger.error('Uncaught Exception:', error);
  process.exit(1);
});

process.on('unhandledRejection', (error) => {
  logger.error('Unhandled Rejection:', error);
  process.exit(1);
});

process.on('SIGTERM', () => {
  logger.info('SIGTERM received');
  process.exit(0);
});